@@ -7,7 +7,14 @@ use Module::Runtime qw(require_module);
 use Moo;
 use 5.10.1;
 
-our $VERSION = '0.0.14';
+our $VERSION = '0.0.15';
+
+BEGIN {
+    if (my $trace_var = $ENV{PERL_ZOOKEEPER_TRACE}) {
+        my ($level, $file) = split /=/, $trace_var;
+        __PACKAGE__->trace($level, $file);
+    }
+}
 
 =head1 NAME
 
@@ -130,14 +137,30 @@ has timeout => (
 
 =head2 watcher
 
-A subroutine reference to be called by a default watcher for ZooKeeper session events.
+A subroutine reference to be called by a default watcher for ZooKeeper session events. This attribute is read/write.
 
 =cut
 
 has watcher => (
-    is => 'ro',
+    is => 'rw',
 );
 
+after watcher => sub {
+    # this probably makes more sense as a trigger
+    #  but triggers get run before BUILD
+    #  which is a problem for watch instantiation
+    return if @_ == 1;
+    my ($self, $code) = @_;
+    my ($path, $type) = ('', 'default');
+    my %watcher_args  = (path => $path, type => $type);
+
+    my @old = $self->get_watchers(%watcher_args);
+    my $new = $self->create_watcher($path => $code, type => $type);
+    $self->_set_watcher($new);
+    $self->remove_watcher(%watcher_args, watcher => $_) for @old;
+};
+
+
 =head2 authentication
 
 An arrayref used for authenticating with ZooKeeper. This will be passed as an array to add_auth.
@@ -193,7 +216,12 @@ has _dispatcher_obj => (
     init_arg => undef,
     lazy     => 1,
     builder  => '_build_dispatcher_obj',
-    handles  => [qw(create_watcher wait)],
+    handles  => [qw(
+        create_watcher
+        get_watchers
+        remove_watcher
+        wait
+    )],
 );
 
 sub _build_dispatcher_obj {
@@ -223,7 +251,7 @@ has ignore_session_events => (
 
 sub BUILD {
     my ($self, $args) = @_;
-    my $default_watcher = $self->watcher ? $self->create_watcher('' => $self->watcher, type => 'default') : undef;
+    my $default_watcher = $args->{watcher} ? $self->create_watcher('' => $args->{watcher}, type => 'default') : undef;
 
     $self->_xs_init($self->hosts, $self->timeout, $default_watcher, $args->{client_id});
 
@@ -442,6 +470,17 @@ around set_acl => sub {
     return $self->$orig($path, $acl, $extra{version}//-1);
 };
 
+=head2 trace
+
+Set the tracing level for the ZooKeeper client. Can also be set using the PERL_ZOOKEEPER_TRACE environmental variable, where PERL_ZOOKEEPER_TRACE=$level=$file traces to $file with debug level $level.
+
+    $zk->trace($level, $file)
+
+        REQUIRED $level
+        OPTIONAL $file
+
+=cut
+
 =head1 CAVEATS
 
 =head2 Forking
